package yrpg.fi.cas.plugin.other.webapi;

import com.alibaba.fastjson.JSONObject;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.openapi.common.custom.annotation.ApiController;
import kd.bos.openapi.common.custom.annotation.ApiParam;
import kd.bos.openapi.common.custom.annotation.ApiPostMapping;
import kd.bos.openapi.common.result.CustomApiResult;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import yrpg.fi.cas.plugin.other.util.BRApiReqModel;

import javax.validation.Valid;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

@ApiController(value="CommonBill",desc="统一url入口")
public class PaymentProcessingCallbacksApi {
    //存放实际地址和接口入口的
    @ApiPostMapping(value="paymentProcessingCallbacks")
    public CustomApiResult<Object> paymentProcessingCallbacks(@Valid @ApiParam("入口参数") BRApiReqModel list) {
        //获取入参
        String detailstr=list.getDetail();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        //xml解析为JSONObject
        JSONObject detail=JSONObject.parseObject(detailstr);

        long id=detail.getLong("SRCSERIALNO");//来源单据id
        String fundnumber=detail.getString("NOTECODE");//资金系统单据号
        String paystatus=detail.getString("PAYSTATE");//支付状态
        String bankreturnmsg=detail.getString("PAYINFO");//银行返回信息

        String lastpaybank=detail.getString("LASTPAYBANK");//实际付方银行名称
        QFilter brfinancialcodeFilter = new QFilter("yrpg_brfinancialcode", QCP.equals, lastpaybank);//保融银行代码
        QFilter[] bankfilters = new QFilter[]{brfinancialcodeFilter};
        DynamicObject bankcgsetting=null;
        if(lastpaybank!=null&&!"".equals(lastpaybank)){
            try{
                bankcgsetting = BusinessDataServiceHelper.loadSingle("bd_bankcgsetting","id,number,name",bankfilters);
            }catch (Exception e){
                String response="<list><detail><SRCSERIALNO>"+id+"</SRCSERIALNO><DEALSTATE>E</DEALSTATE><DEALINFO>保融银行代码"+lastpaybank+"未维护</DEALINFO></detail></list>";
                return CustomApiResult.success(response);
            }
        }

        String lastaccountnumber=detail.getString("LASTACCOUNTNUMBER");//实际付方账号

        Date paydate=null;
        if(detail.getString("PAYMADEDATE")!=null&&!"".equals(detail.getString("PAYMADEDATE"))){
            try {
                paydate=sdf.parse(detail.getString("PAYMADEDATE"));//付款日期
            } catch (ParseException e) {
                throw new RuntimeException(e);
            }
        }

        BigDecimal actualamount=detail.getBigDecimal("LASTOURAMOUNT");//实付金额
        String bankcheckflag=detail.getString("CHECKCODE");//付款码

        DynamicObject caspaybill=null;

        try{
            caspaybill= BusinessDataServiceHelper.loadSingle(id,"cas_paybill");
        }catch (Exception e){
            String response="<list><detail><SRCSERIALNO>"+id+"</SRCSERIALNO><DEALSTATE>E</DEALSTATE><DEALINFO>付款处理单不存在</DEALINFO></detail></list>";
            return CustomApiResult.success(response);
        }

        //版本号
        int brcode=caspaybill.getInt("yrpg_brcode");

        if(caspaybill!=null) {
            caspaybill.set("yrpg_fundnumber", fundnumber);
            caspaybill.set("bankreturnmsg", bankreturnmsg);
            if (paydate != null) {
                caspaybill.set("paydate", paydate);
            }
            caspaybill.set("yrpg_actualamount", actualamount);
            caspaybill.set("bankcheckflag", bankcheckflag);

            //回调支付状态参数设置
            if ("2".equals(paystatus)) {//支付成功
                caspaybill.set("yrpg_paystatus", "2");
                caspaybill.set("billstatus", "C");
                caspaybill.set("yrpg_actualypayaccout",lastaccountnumber);
                if(bankcgsetting!=null){
                    caspaybill.set("yrpg_actualpaybank",bankcgsetting);
                }
                //更新付款处理单
                SaveServiceHelper.update(caspaybill);
            } else if ("3".equals(paystatus)) {//支付失败
                caspaybill.set("yrpg_paystatus", "3");
                caspaybill.set("billstatus", "G");
                brcode++;
                caspaybill.set("yrpg_brcode",brcode);
                //更新付款处理单
                SaveServiceHelper.update(caspaybill);
            } else if ("6".equals(paystatus)) {//退票
                caspaybill.set("yrpg_paystatus", "5");
                caspaybill.set("billstatus", "G");
                brcode++;
                caspaybill.set("yrpg_brcode",brcode);
                //更新付款处理单
                SaveServiceHelper.update(caspaybill);
            }
        }

        String response="<list><detail><SRCSERIALNO>"+id+"</SRCSERIALNO><DEALSTATE>S</DEALSTATE></detail></list>";
        return CustomApiResult.success(response);
    }
}
